function [x, y] = parseFDB(input)
% parseFDB: parses fragment database. After parsing stores it
% in a global variable.
% Parameters:
%  input = the database as a (line)-struct array

% Mika Miettinen (based on earlier version by Timo Kriinen)

% Different error codes and their meanings
% -21  First column is wrong
% -22  Second column is wrong
% -23  Third column is wrong
% -24  Fourth column is wrong
% -25  Fifth column is wrong
% -26  Sixth column is wrong
% -27  End of the line contains garbage
% -29  File contains no datalines
% -28  Mass of the mother ion is not equal to the mass given in initializeGlobals.m

    global AminoAcids;

    accepted = 0;  % number of accepted lines (correct datalines)

    for i = 1 : length(input)
        tline = input(i).line;
        [token, rem] = strtok(tline);
        % discard comments and empty lines
        if isempty(token)
            continue;
        elseif token(1) == '%'
            continue;
        end

        % first token must be a valid shorthand for an amino acid
        acidIdx = findAminoAcid(token);
        if acidIdx == 0
            x = -21;
            y = i;
            return;
        else
            fragment.molecule = token;
        end

        % the second column must be an 'M' or 'D'
        [mdStr, rem] = strtok(rem);
        if ~(upper(mdStr) == 'M' | upper(mdStr) == 'D')
            x = -22;
            y = i;
            return;
        else
            fragment.role = upper(mdStr);
        end

        % the third column must have a valid atomic formula in it
        [formula, rem] = strtok(rem);
        frm = parseFormula2(formula);
        if isempty(formula) | isempty(frm)
            x = -23;
            y = i;
            return;
        else
            fragment.formula = frm;
        end

        % then we must have a valid carbon chain
        [carbonStr, rem] = strtok(rem);
        carbonsOK = parseCarbons(carbonStr);
        if ~carbonsOK
            x = -24;
            y = i;
            return;
        else
            fragment.carbons = carbonStr;
        end

        % the fifth column must be an m/z value
        [mperzStr, rem] = strtok(rem);
        if ~isNumber(mperzStr)
            x = -25;
            y = i;
            return;
        else
            mperz = round(str2num(mperzStr));
            if mperz < 1
                x = -25;
                y = i;
                return;
            end
            if fragment.role == 'M' & mperz ~= AminoAcids(acidIdx).mass %sanity check
                x = -28;
                y = i;
                return;
            end
            fragment.mz = mperz;
        end

        % the sixth column is optional; if it is empty, this fragment
        % is a mother, otherwise it denotes the relative strength of daughter
        [strengthStr,rem] = strtok(rem);
        if (isempty(strengthStr) | strengthStr(1) == '%') & fragment.role == 'M'
            fragment.probability = NaN;
        elseif isNumber(strengthStr)
            strength = str2num(strengthStr);
            if strength < 0 | strength > 100
                x = -26;
                y = i;
                return;
            end
            fragment.probability = strength;
        else
            x = -26;
            y = i;
            return;
        end

        token = strtok(rem);
        %if ~isempty(token) & token(1) ~= '%'
            % the rest is garbage; doesn't belong here
            % AR: THIS DOES NOT WOTK CORRECTLY IF THE COMMENT IS IN THE MOTHER ION!!
         %   x = -27;
         %   y = i;
         %   return;
         %end

        % since we are here, everything must be OK (in this line)
        AminoAcids(acidIdx).fragments = cat(1, AminoAcids(acidIdx).fragments, fragment);
        accepted = accepted + 1;
    end

    if accepted == 0
        x = -29;  % no lines were accepted
        y = 0;
        return;
    end

    x = 1;
    y = length(input)+1;
