function [x, y] = parseInput(input)
% parseInput: parses the input for syntax errors.
% Stores parsed input in a global array.
% Parameters:
%  input = the input as a (line)-struct array

% Mika Miettinen (based on earlier version by Timo Kriinen)

% Error codes and their meanings:
% -11 First column was wrong
% -12 Second column was wrong
% -13 Third column was wrong
% -14 End of the line contained garbage
% -15 File contained no datalines

    global InputArray;  % a global struct-array for storing the input
    global InputComments; % global array for comments at the beginning of input
    global InputDatalines; % global array containing datalines of the input

    accepted = 0;  % number of lines accepted (correct datalines)
    scanningComments = 1;

    for i = 1 : length(input)
        tline = input(i).line;
        inputLine = struct('line', tline);
        [token, rem] = strtok(tline);
        % comments appearing at the beginning of input are appended
        % to Comments-array; comments appearing elsewhere are discarded
        if isempty(token)
            if length(InputComments) ~= 0 & ~isempty(InputArray)
                scanningComments = 0;
            end
            continue;
        elseif token(1) == '%' & scanningComments == 1
            InputComments = cat(1, InputComments, inputLine);
            continue;
        elseif token(1) == '%' & scanningComments == 0
            continue;
        else
            scanningComments = 0;
        end


        % first token must be a number
        if ~isNumber(token)
            x = -11;
            y = i;
            return;
        else
            mperz = round(str2num(token));
            if mperz < 1
                x = -11;
                y = i;
                return;
            end
            peak.mz = mperz;
        end

        % the second column must have a number in it too
        [intensityStr, rem] = strtok(rem);
        if ~isNumber(intensityStr)
            x = -12;
            y = i;
            return;
        else
            int = round(str2num(intensityStr));
            if int < 1
                x = -12;
                y = i;
                return;
            end
            peak.intensity = int;
        end

        % the third column is optional;
        % it must be an 'M', a number or empty
        [motherStr, rem] = strtok(rem);
        if isempty(motherStr)
            peak.mother = Inf;
        elseif upper(motherStr) == 'M'
            peak.mother = NaN;
        elseif isNumber(motherStr)
            mutter = round(str2num(motherStr));
            if mutter < 0
                x = -13;
                y = i;
                return;
            end
            peak.mother = mutter;
        else
            x = -13;
            y = i;
            return;
        end

        token = strtok(rem);
        if ~isempty(token) & token(1) ~= '%'
            % the rest is garbage; shouldn't be here
            x = -14;
            y = i;
            return;
        end

        % since we are here, everything is OK; append struct
        InputArray = cat(1, InputArray, peak);
        InputDatalines = cat(1, InputDatalines, inputLine);
        accepted = accepted + 1;

    end

    if accepted == 0
        % no proper datalines; clear InputComments
        InputComments = struct('line', {});
        x = -15; % no lines were accepted
        y = 0;
        return;
    end

    x = 1;
    y = length(input) + 1;


